//*****************************************************************************
//
//      Function:       GetTokenInformation
//      Author:         Kenny Kerr
//      Description:    Wrapper for the system's GetTokenInformation function
//                      to simplify memory allocations.
//
//*****************************************************************************

#pragma once

#include "LocalMemory.h"

namespace Kerr
{
    template <typename InfoType>
    HRESULT GetTokenInformation(HANDLE token,
                                TOKEN_INFORMATION_CLASS infoClass,
                                LocalMemory<InfoType*>& info)
    {
        ASSERT(0 != token);
        DWORD size = 0;

        HRESULT result = S_OK;

        if (!::GetTokenInformation(token,
                                   infoClass,
                                   0,
                                   0,
                                   &size))
        {
            const DWORD lastError = ::GetLastError();

            if (ERROR_INSUFFICIENT_BUFFER != lastError)
            {
                result = HRESULT_FROM_WIN32(lastError);
            }
        }

        if (SUCCEEDED(result))
        {
            LocalMemory<InfoType*> localMemory;
            result = localMemory.Allocate(size);

            if (SUCCEEDED(result))
            {
                if (!::GetTokenInformation(token,
                                           infoClass,
                                           localMemory.m_p,
                                           size,
                                           &size))
                {
                    result = HRESULT_FROM_WIN32(::GetLastError());
                }
                else
                {
                    info.Attach(localMemory.Detach());
                }
            }
        }

        return result;
    }
}
