//*****************************************************************************
//
//      Class:          WellKnownSid
//      Author:         Kenny Kerr
//      Description:    Simple wrapper for local memory allocations, useful
//                      for security programming.
//
//*****************************************************************************

#pragma once

namespace Kerr
{
    template <typename P>
    class LocalMemory
    {
    public:

        LocalMemory() :
            m_p(0)
        {
        }

        ~LocalMemory()
        {
            Free();
        }

        HRESULT Allocate(size_t bytes)
        {
            Free();

            m_p = static_cast<P>(::LocalAlloc(LPTR,
                                              bytes));

            return 0 != m_p ? S_OK : HRESULT_FROM_WIN32(::GetLastError());
        }

        void Attach(P p)
        {
            Free();
            m_p = p;
        }

        P Detach()
        {
            P result = m_p;
            m_p = 0;
            return result;
        }

        void Free()
        {
            if (0 != m_p)
            {
                ::LocalFree(m_p);
                m_p = 0;
            }
        }

        P operator->() const
        {
            ASSERT(0 != m_p);
            return m_p;
        }

        P m_p;

    private:

        LocalMemory(const LocalMemory&);
        LocalMemory& operator=(const LocalMemory&);

    };
}
