//*****************************************************************************
//
//      Class:          WellKnownSid
//      Author:         Kenny Kerr
//      Description:    Utility class to easily form "well-known" SIDs.
//
//*****************************************************************************

#pragma once

namespace Kerr
{
    struct WellKnownSid : SID
    {
    public:

        WellKnownSid(BYTE authority,
                     DWORD firstSubAuthority,
                     DWORD secondSubAuthority = 0)
        {
            ::ZeroMemory(this, 
                         sizeof (WellKnownSid));

            Revision = SID_REVISION;
            SubAuthorityCount = (0 != secondSubAuthority ? 2 : 1);
            IdentifierAuthority.Value[5] = authority;
            SubAuthority[0] = firstSubAuthority;
            SubAuthority[1] = secondSubAuthority;
        }

        BYTE GetAuthority() const
        {
            return IdentifierAuthority.Value[5];
        }

        DWORD GetFirstSubAuthority() const
        {
            return SubAuthority[0];
        }

        DWORD GetSecondSubAuthority() const
        {
            return SubAuthority[1];
        }

        static WellKnownSid Everyone()
        {
            return WellKnownSid(WorldAuthority,
                                SECURITY_WORLD_RID);
        }

        static WellKnownSid Administrators()
        {
            return WellKnownSid(NtAuthority,
                                SECURITY_BUILTIN_DOMAIN_RID,
                                DOMAIN_ALIAS_RID_ADMINS);
        }

        static WellKnownSid Users()
        {
            return WellKnownSid(NtAuthority,
                                SECURITY_BUILTIN_DOMAIN_RID,
                                DOMAIN_ALIAS_RID_USERS);
        }

        enum WellKnownAuthorities
        {
            NullAuthority,
            WorldAuthority,
            LocalAuthority,
            CreatorAuthority,
            NonUniqueAuthority,
            NtAuthority,
            MandatoryLabelAuthority = 16
        };

    private:

        DWORD m_secondSubAuthority;
    };
}
